package remotestocks;

import java.util.Hashtable;

// This class implements a remote server for the StockQuoteServer interface.
// It keeps a table of stocks, and a table of clients indexed by stock.
// When a client wants to receive stock quotes it calls addWatch.
// When a quote is updated, this object invokes the update method
// in every client that is watching that stock.

// Since this program is just a demo, the list of stocks is
// hard-coded and the stock values are updated every 5 seconds
// by a random amount.

public class RemoteStockQuoteImpl
	implements StockQuoteServerServant
{
// the table of clients, used for mapping from a stocks.StockQuoteClient
// to a remotestocks.StockQuoteClientRef
	protected Hashtable stockClients;

	static int servicePort = 5150;
	static String pathName = "StockQuotes";

	protected stocks.StockQuoteServerImpl impl;

	public RemoteStockQuoteImpl(stocks.StockQuoteServerImpl impl)
	{
		this.impl = impl;
		stockClients = new Hashtable();
	}

	protected void removeCallback(RemoteStockCallback callback)
	{
		stockClients.remove(callback.client);
	}

// addWatch adds a client to the list of clients watching a stock

	public void addWatch(StockQuoteClientRef client, String stock)
	throws sunw.corba.SystemException, UnknownStock
	{

// See if we have already created a callback object for this client. 
		RemoteStockCallback callback = (RemoteStockCallback)
			stockClients.get(client);

// If we didn't already have a callback, create one and add it
// to the table.
		if (callback == null) {
			callback = new RemoteStockCallback(client, this);

			stockClients.put(client, callback);
		}

// Now call the addWatch method in the real implementation
		try {
			impl.addWatch(callback, stock);
		} catch (stocks.UnknownStockException e) {
			throw new UnknownStock(stock);
		}
	}

// removeWatch removes a client from the list of clients watching a stock.

	public void removeWatch(StockQuoteClientRef client, String stock)
	throws sunw.corba.SystemException, UnknownStock
	{

		RemoteStockCallback callback = (RemoteStockCallback)
			stockClients.get(client);

		if (callback == null) return;

		try {
			impl.removeWatch(callback, stock);
		} catch (stocks.UnknownStockException e) {
			throw new UnknownStock(stock);
		}
	}

// removeClient removes a client from each list of watched stocks

	public void removeClient(StockQuoteClientRef client)
	throws sunw.corba.SystemException
	{
		RemoteStockCallback callback = (RemoteStockCallback)
			stockClients.get(client);

		if (callback == null) return;
		impl.removeClient(callback);
	}

// getStockList returns an array of all the available stocks

	public StockInfo getStockList()
	throws sunw.corba.SystemException
	{

		String[] stockList = impl.getStockList();

		return new StockInfo(stockList);
	}

	public static void main(String[] args)
	{

		sunw.door.Orb.initialize(servicePort);

		try {

			stocks.StockQuoteServerImpl realImpl =
				new stocks.StockQuoteServerImpl();

			RemoteStockQuoteImpl impl =
				new RemoteStockQuoteImpl(realImpl);
// Create the server
			StockQuoteServerRef server =
				StockQuoteServerSkeleton.createRef(impl);

			Thread stockThread = new Thread(realImpl);
			stockThread.start();

// Register the object with the naming service as "StockQuotes"
			sunw.door.Orb.publish(pathName, server);

		} catch (Exception e) {
			System.out.println("Got exception: "+e);
			e.printStackTrace();
		}
	}
}
