package banking;

import java.util.*;

// This class implements a remote banking object. It sets up
// a set of dummy accounts and allows you to manipulate them
// through the Banking interface.
//
// Accounts are identified by the combination of the account id,
// the password and the account type. This is a quick and dirty
// way to work, and not the way a bank would normally do it, since
// the password is not part of the unique identifier of the account.

public class BankingImpl extends _sk_Banking
{
	public Hashtable accountTable;

// The constructor creates a table of dummy accounts.

	public BankingImpl(String serviceName)
	{
		super(serviceName);

		accountTable = new Hashtable();

		accountTable.put(
			new Account("AA1234", "1017", AccountType.CHECKING),
			new Integer(50000));	// $500.00 balance

		accountTable.put(
			new Account("AA1234", "1017", AccountType.SAVINGS),
			new Integer(148756));	// $1487.56 balance

		accountTable.put(
			new Account("AB5678", "4456", AccountType.CHECKING),
			new Integer(7742));	// $77.32 balance

		accountTable.put(
			new Account("AB5678", "4456", AccountType.SAVINGS),
			new Integer(32201));	// $322.01 balance
	}

// getBalance returns the amount of money in the account (in cents).
// If the account is invalid, it throws an InvalidAccountException

	public int getBalance(AccountInfo accountInfo)
	throws CORBA.SystemException, InvalidAccountException
	{

// Fetch the account from the table
		Integer balance = (Integer) accountTable.get(
			new Account(accountInfo));

// If the account wasn't there, throw an exception
		if (balance == null) {
			throw new InvalidAccountException(accountInfo);
		}

// Return the account's balance
		return balance.intValue();
	}

// withdraw subtracts an amount from the account's balance. If
// the account is invalid, it throws InvalidAccountException.
// If the withdrawal amount exceeds the account balance, it
// throws InsufficientFundsException.

	public synchronized void withdraw(AccountInfo accountInfo, int amount)
	throws CORBA.SystemException, InvalidAccountException,
		InsufficientFundsException
	{

		Account account = new Account(accountInfo);

// Fetch the account
		Integer balance = (Integer) accountTable.get(account);

// If the account wasn't there, throw an exception
		if (balance == null) {
			throw new InvalidAccountException(accountInfo);
		}

// If we are trying to withdraw more than is in the account,
// throw an exception

		if (balance.intValue() < amount) {
			throw new InsufficientFundsException();
		}

// Put the new balance in the account

		accountTable.put(account, new Integer(balance.intValue() -
			amount));
	}

// Deposit adds an amount to an account. If the account is invalid
// it throws an InvalidAccountException

	public synchronized void deposit(AccountInfo accountInfo, int amount)
	throws CORBA.SystemException, InvalidAccountException
	{

		Account account = new Account(accountInfo);

// Fetch the account
		Integer balance = (Integer) accountTable.get(account);

// If the account wasn't there, throw an exception
		if (balance == null) {
			throw new InvalidAccountException(accountInfo);
		}

// Update the account with the new balance
		accountTable.put(account, new Integer(balance.intValue() +
			amount));
	}

// Transfer subtracts an amount from fromAccount and adds it to toAccount.
// If either account is invalid it throws InvalidAccountException.
// If there isn't enough money in fromAccount it throws
// InsufficientFundsException.

	public synchronized void transfer(AccountInfo fromAccountInfo,
		AccountInfo toAccountInfo, int amount)
	throws CORBA.SystemException, InvalidAccountException,
		InsufficientFundsException
	{
		Account fromAccount = new Account(fromAccountInfo);
		Account toAccount = new Account(toAccountInfo);

// Fetch the from account
		Integer fromBalance = (Integer) accountTable.get(fromAccount);

// If the from account doesn't exist, throw an exception
		if (fromBalance == null) {
			throw new InvalidAccountException(fromAccountInfo);
		}

// Fetch the to account
		Integer toBalance = (Integer) accountTable.get(toAccount);

// If the to account doesn't exist, throw an exception
		if (toBalance == null) {
			throw new InvalidAccountException(toAccountInfo);
		}

// Make sure the from account contains enough money, otherwise throw
// an InsufficientFundsException.

		if (fromBalance.intValue() < amount) {
			throw new InsufficientFundsException();
		}

		
// Subtract the amount from the fromAccount
		accountTable.put(fromAccount,
			new Integer(fromBalance.intValue() - amount));

// Add the amount to the toAccount
		accountTable.put(toAccount,
			new Integer(toBalance.intValue() + amount));
	}
}

